EXPECTED_SONAR_JSON_COMPONENTS = [
    {
        "id": "AX9GDsKlZuVL7NjXSAZ4",
        "key": "fga-eps-mds_2021-2-MeasureSoftGram-CLI:tests/__init__.py",
        "name": "__init__.py",
        "qualifier": "UTS",
        "path": "tests/__init__.py",
        "language": "py",
        "measures": [
            {"metric": "security_rating", "value": "1.0", "bestValue": True},
            {"metric": "test_errors", "value": "0", "bestValue": True},
            {"metric": "duplicated_lines_density", "value": "0.0", "bestValue": True},
            {"metric": "test_failures", "value": "0", "bestValue": True},
        ],
    },
    {
        "id": "AX9Fg6R0WRlRH47OejaP",
        "key": "fga-eps-mds_2021-2-MeasureSoftGram-CLI:measuresoftgram/cli.py",
        "name": "cli.py",
        "qualifier": "FIL",
        "path": "measuresoftgram/cli.py",
        "language": "py",
        "measures": [
            {"metric": "complexity", "value": "2"},
            {"metric": "functions", "value": "2"},
            {"metric": "ncloc", "value": "4"},
            {"metric": "coverage", "value": "100.0", "bestValue": True},
            {"metric": "security_rating", "value": "1.0", "bestValue": True},
            {"metric": "comment_lines_density", "value": "20.0", "bestValue": False},
            {"metric": "files", "value": "1"},
            {"metric": "test_errors", "value": "0", "bestValue": True},
            {"metric": "duplicated_lines_density", "value": "0.0", "bestValue": True},
            {"metric": "test_failures", "value": "0", "bestValue": True},
        ],
    },
    {
        "id": "AX9GDsKlZuVL7NjXSAZ3",
        "key": "fga-eps-mds_2021-2-MeasureSoftGram-CLI:tests/hello_world_test.py",
        "name": "hello_world_test.py",
        "qualifier": "UTS",
        "path": "tests/hello_world_test.py",
        "language": "py",
        "measures": [
            {"metric": "security_rating", "value": "1.0", "bestValue": True},
            {"metric": "test_execution_time", "value": "2"},
            {"metric": "tests", "value": "2"},
            {"metric": "test_errors", "value": "0", "bestValue": True},
            {"metric": "duplicated_lines_density", "value": "0.0", "bestValue": True},
            {"metric": "test_failures", "value": "0", "bestValue": True},
        ],
    },
    {
        "id": "AX9Fg6R0WRlRH47OejaQ",
        "key": "fga-eps-mds_2021-2-MeasureSoftGram-CLI:measuresoftgram",
        "name": "measuresoftgram",
        "qualifier": "DIR",
        "path": "measuresoftgram",
        "measures": [
            {"metric": "duplicated_lines_density", "value": "0.0", "bestValue": True},
            {"metric": "functions", "value": "2"},
            {"metric": "security_rating", "value": "1.0", "bestValue": True},
            {"metric": "files", "value": "1"},
            {"metric": "complexity", "value": "2"},
            {"metric": "ncloc", "value": "4"},
            {"metric": "coverage", "value": "100.0", "bestValue": True},
            {"metric": "comment_lines_density", "value": "20.0", "bestValue": False},
        ],
    },
    {
        "id": "AX9GDsKlZuVL7NjXSAZ5",
        "key": "fga-eps-mds_2021-2-MeasureSoftGram-CLI:tests",
        "name": "tests",
        "qualifier": "DIR",
        "path": "tests",
        "measures": [
            {"metric": "test_execution_time", "value": "2"},
            {"metric": "test_failures", "value": "0", "bestValue": True},
            {"metric": "test_errors", "value": "0", "bestValue": True},
            {"metric": "security_rating", "value": "1.0", "bestValue": True},
            {"metric": "tests", "value": "2"},
        ],
    },
    {
        "id": "AX9FgyLHNIj_v_uQK41e",
        "key": "fga-eps-mds_2021-2-MeasureSoftGram-CLI",
        "name": "2021-2-MeasureSoftGram-CLI",
        "qualifier": "TRK",
        "measures": [
            {"metric": "duplicated_lines_density", "value": "0.0", "bestValue": True},
            {"metric": "functions", "value": "2"},
            {"metric": "test_execution_time", "value": "2"},
            {"metric": "test_failures", "value": "0", "bestValue": True},
            {"metric": "test_errors", "value": "0", "bestValue": True},
            {"metric": "security_rating", "value": "1.0", "bestValue": True},
            {"metric": "tests", "value": "2"},
            {"metric": "files", "value": "1"},
            {"metric": "complexity", "value": "2"},
            {"metric": "ncloc", "value": "4"},
            {"metric": "coverage", "value": "100.0", "bestValue": True},
            {"metric": "comment_lines_density", "value": "20.0", "bestValue": False},
        ],
    },
]
